;GO_X   EQU  0
;GO_Y   EQU  1
;GO_Z   EQU  2
;X_STEP EQU  3
;Y_STEP EQU  4
;Z_STEP EQU  5
;X_DIR  EQU  6
;Y_DIR  EQU  7
;Z_DIR  EQU  8
;X_SIZE EQU  20h
;Y_SIZE EQU  22h
;Z_SIZE EQU  24h
;DELTA  EQU  26h
;DELTB  EQU  28h


; implementation of Breshenams DDA line drawing algorithm for an 8051
; note that this code is in line for speed and requires duplication
; hence the macros
; this is designed to drive steppers so direction is set externally
; before stepping commences
; Algorithm asssumes that the longest vector is already determined
; Also note that this has been extended to do 3D lines
; (GO_X is set if |x|>=|y| && |x|>=|z|
;    for count = x; count > 0; count --
;    {
;           start with deltam = 0
;           step x
;           deltam += y
;           temp = deltam -x
;           if temp >= 0
;           {
;              step y
;              deltam = temp
;           }
;    }
; the implementation follows this generally but note that x & y remain
; cleared as long as possible to give decent pulses widths
;
; The following must be defined in the calling routine
ADD_2    MACRO   dest,source
        MOV     A,dest+1
        ADD     A,source+1
        MOV     dest+1,a
        MOV     A,dest
        ADDC    A,source
        MOV     dest,a
        endm

INC_2    MACRO   dest
        MOV     A,dest+1
        add     a,#1
        mov     dest+1,a
        mov     a,dest
        addc    a,#0
        mov     dest,a
        endm
CMP_2 .macro am,bm
       CLR     C
       MOV     A,am+1
       SUBB    A,bm+1
       MOV     A,am
       SUBB    A,bm     ; CY set iff B > A
       endm
SUB_2 .macro am,bm
       CLR     C
       MOV     A,am+1
       SUBB    A,bm+1
       MOV     am+1,A
       MOV     A,am
       SUBB    A,bm
       MOV     am,a
       endm
NEG_2  .macro  prim_sz
       CLR     C
       CLR     A
       SUBB    A,prim_sz+1
       MOV     prim_sz+1,A
       CLR     A
       SUBB    A,prim_sz
       MOV     prim_sz,A
       endm

BRESH   MACRO  prim_step,sec_step,sec1_step,prim_sz,sec_sz,sec1_sz
        SETB    prim_step               ; step on MAX axis
        ADD_2    DELTA,sec_sz           ; add secondary size to delta
        CMP_2    DELTA,prim_sz          ; if delta >= primary size No Carry
        jc      $1
        SETB    sec_step                ; step on minor axis
        SUB_2    DELTA,prim_sz          ; subtract primary size from delta
$1:
        ADD_2    DELTB,sec1_sz          ; add alt secondary to delta2
        CMP_2    DELTB,prim_sz          ; if delta2 >= primary size No Carry
        jc      $2
        SETB    sec1_step               ; setp on minor axis
        SUB_2    DELTB,prim_sz          ; subtract primary size from delta
$2:
        endm
TEST_DIR   .macro  sz,dir,label
       MOV     A,sz
       JNB     ACC.7,label
       SETB    dir    ; -ve step dir
       NEG_2   sz
label:
      endm

BRESH_X:
        BRESH   X_STEP,Y_STEP,Z_STEP,X_SIZE,Y_SIZE,Z_SIZE
        RET
DO_BRESH:
        JB   GO_X,BRESH_X
        JB   GO_Y,BRESH_Y
        JB   GO_Z,BRESH_Z
        RET
BRESH_Y:
        BRESH   Y_STEP,X_STEP,Z_STEP,Y_SIZE,X_SIZE,Z_SIZE
        RET
BRESH_Z:
        BRESH   Z_STEP,X_STEP,Y_STEP,Z_SIZE,X_SIZE,Y_SIZE
        RET
__xyz:
       CLR     FLAG_1
       SETB    FLAG_2
       SETB    FLAG_3
       CLR     X_DIR
       CLR     Y_DIR
       CLR     Z_DIR
       CLR     GO_X
       CLR     GO_Y
       CLR     GO_Z

       CLR     A
       MOV     CV,A
       MOV     CCNT,A
       MOV     CCNT+1,A
       MOV     DELTA,A
       MOV     DELTA+1,A
       MOV     DELTB,A
       MOV     DELTB+1,A

       .if TESTING
       MOV     __TESTX,A
       MOV     __TESTX+1,A
       MOV     __TESTY,A
       MOV     __TESTY+1,A
       MOV     __TESTZ,A
       MOV     __TESTZ+1,A
       .endif

       MOV     ATX,A
       MOV     CT,A
       MOV     CT+1,A
       MOV     ACL,#1

       ORL     A,X_SIZE
       ORL     A,X_SIZE+1
       ORL     A,Y_SIZE
       ORL     A,Y_SIZE+1
       ORL     A,Z_SIZE
       ORL     A,Z_SIZE+1
       JNZ     DO_MORE
       RET
DO_MORE:
       TEST_DIR  X_SIZE,X_DIR,td0
       TEST_DIR  Y_SIZE,Y_DIR,td1
       TEST_DIR  Z_SIZE,Z_DIR,td2

;                       ; now find greater of X & Y & Z to generate N
       CMP_2   X_SIZE,Y_SIZE
       jc      Y_IS_GREATER_THAN_X
       CMP_2   X_SIZE,Z_SIZE
       jc      Z_IS_GREATER_THAN_X
X_IS_HIGHEST:
       MOV     CT,X_SIZE
       MOV     CT+1,X_SIZE+1
       SETB    GO_X
       JMP     FOUND_GREATEST

Y_IS_GREATER_THAN_X:
       CMP_2   Y_SIZE,Z_SIZE
       jc      Z_IS_HIGHEST
Y_IS_HIGHEST:
       MOV     CT,Y_SIZE
       MOV     CT+1,Y_SIZE+1
       SETB    GO_Y
       JMP     FOUND_GREATEST
Z_IS_GREATER_THAN_X:
       CMP_2   Z_SIZE,Y_SIZE
       jc      Y_IS_HIGHEST
Z_IS_HIGHEST:
       MOV     CT,Z_SIZE
       MOV     CT+1,Z_SIZE+1
       SETB    GO_Z
FOUND_GREATEST:
                   ; given N steps load counters for accell/decell

       MOV     B,CT+1    ; low in B high in a
       MOV     A,CT

       CLR     C
       RRC     A         ; high /= 2
       XCH     A,B       ; high->b , low->a
       RRC     A         ; low +c /=2  INT(N/2) in B:A
       MOV     ATX,A     ; if < 255 stick it in ATX
       XCH     A,B       ; Check high byte
       JZ      B_BELOW_MAX ; if something here
       MOV     ATX,#0ffh ; max accel / decel time = 255
B_BELOW_MAX:
;
       MOV     A,ATX     ; get accel time
       MOV     B,#2
       MUL     AB
       CLR     C
       XCH     A,CT+1      ; counts = counts - 2 * Accel
       SUBB    A,CT+1
       MOV     CT+1,A
       MOV     A,CT
       SUBB    A,B
       MOV     CT,A      ; load remainder into cruise time

       MOV     CCNT+1,ATX  ; pre-load initial accel CCNT here
       MOV     CCNT,#0    ;

       SETB    FLAG_1    ; startit
       RET

